SystemOrganization addCategory: #'MonticelloFileTree-Git'!

FileUrl subclass: #GitFileTreeUrl
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Git'!

!GitFileTreeUrl commentStamp: '<historical>' prior: 0!
A FileTreeUrl represent a gitfiletree:// path for Gofer.!

!GitFileTreeUrl classSide methodsFor: 'constants' stamp: 'ThierryGoubier 6/9/2013 13:48'!
schemeName
	^ 'gitfiletree'! !

!GitFileTreeUrl methodsFor: 'accessing' stamp: 'ThierryGoubier 5/29/2013 16:25'!
mcRepositoryAsUser: usernameString withPassword: passwordString
	"Where we create the repository."

	^ MCFileTreeGitRepository new directory: self asFileReference! !

MCFileTreeStCypressReader subclass: #MCFileTreeGitStReader
	instanceVariableNames: 'commitID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Git'!

!MCFileTreeGitStReader commentStamp: '<historical>' prior: 0!
This class handles reading a package from a git (via a specific commit ID).

Uses OS-Process to call git archive and retrieve a zip of the package. Then sort of reuse the superclass implementation (MczReader?) even if the StCypress format is different.

Remember:
	* stream is the directory holding the .package directory.
	* Package directory is NameOfPackage.package.
	* zip contains the archive of the package (the contents of NameOfPackage.package according to the commitID)
	
Complex work-arounds to handle incorrect incoming streams; see parseMember:!

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 6/5/2013 15:44'!
addClassAndMethodDefinitions
	(self zip members select: [ :m | m isDirectory and: [ m splitFileName last endsWith: '.class' ] ])
		do: [ :entry | self addClassAndMethodDefinitionsFromEntry: entry ].
	(self zip members select: [ :m | m isDirectory and: [ m splitFileName last endsWith: '.extension' ] ])
		do: [ :entry | self addExtensionClassAndMethodDefinitionsFromEntry: entry ]! !

!MCFileTreeGitStReader methodsFor: 'private' stamp: 'ThierryGoubier 6/4/2013 16:46'!
addClassAndMethodDefinitionsFromDirectory: aDirectory
	self shouldNotImplement! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 6/16/2013 15:32'!
addClassAndMethodDefinitionsFromEntry: classEntry
	"A class entry here is a ZipDirectoryMember."

	"Would have been convenient to have directory members listing the files they contain."

	| classPropertiesDict classComment methodPropertiesDict entries |
	entries := self zip members
		select: [ :entry | entry isDirectory not and: [ entry fileName beginsWith: classEntry fileName ] ].
	(entries detect: [ :entry | entry splitFileName last = 'properties.json' ] ifNone: [  ])
		ifNotNil: [ :propertyEntry | classPropertiesDict := MCFileTreeJsonParser parseStream: (self zipEntryContents: propertyEntry) ].
	(entries detect: [ :entry | entry splitFileName last = 'README.md' ] ifNone: [  ])
		ifNotNil: [ :commentEntry | classComment := (self zipEntryContents: commentEntry) contents ].
	methodPropertiesDict := Dictionary new.
	(entries detect: [ :entry | entry splitFileName last = 'methodProperties.json' ] ifNone: [  ])
		ifNotNil: [ :propertyEntry | methodPropertiesDict := MCFileTreeJsonParser parseStream: (self zipEntryContents: propertyEntry) ].
	self addClassDefinitionFrom: classPropertiesDict comment: classComment withSqueakLineEndings.
	self
		addMethodDefinitionsForClass: (classPropertiesDict at: 'name')
		methodProperties: methodPropertiesDict
		in: entries
		root: classEntry! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 6/4/2013 16:37'!
addDefinitionFromEntry: aZipFileMember
	(self definitionFromEntry: aZipFileMember) ifNotNil: [ :def | definitions add: def ]! !

!MCFileTreeGitStReader methodsFor: 'private' stamp: 'ThierryGoubier 6/4/2013 16:38'!
addDefinitionFromFile: directoryEntry inDirectory: aDirectory
	self shouldNotImplement! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 6/16/2013 15:31'!
addExtensionClassAndMethodDefinitionsFromEntry: classEntry
	"A class entry here is a ZipDirectoryMember."

	"Would have been convenient to have directory members listing the files they contain."

	| classPropertiesDict methodPropertiesDict entries |
	entries := self zip members select: [ :entry | entry fileName beginsWith: classEntry fileName ].
	(entries detect: [ :entry | entry splitFileName last = 'properties.json' ] ifNone: [  ])
		ifNotNil: [ :propertyEntry | classPropertiesDict := MCFileTreeJsonParser parseStream: (self zipEntryContents: propertyEntry) ].
	methodPropertiesDict := Dictionary new.
	(entries detect: [ :entry | entry splitFileName last = 'methodProperties.json' ] ifNone: [  ])
		ifNotNil: [ :propertyEntry | methodPropertiesDict := MCFileTreeJsonParser parseStream: (self zipEntryContents: propertyEntry) ].
	self
		addMethodDefinitionsForClass: (classPropertiesDict at: 'name')
		methodProperties: methodPropertiesDict
		in: entries
		root: classEntry! !

!MCFileTreeGitStReader methodsFor: 'private' stamp: 'ThierryGoubier 6/5/2013 10:19'!
addMethodDefinitionsForClass: className methodProperties: methodProperties in: entries
	self shouldNotImplement! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 6/16/2013 12:29'!
addMethodDefinitionsForClass: className methodProperties: methodProperties in: entries root: aClassEntry
	(entries
		select: [ :entry | entry fileName matchesRegex: aClassEntry fileName , '((class)|(instance))' , self slash , '[^' , self slash , ']+.st' ])
		do: [ :methodEntry | 
			| classIsMeta fileStream category source timestamp selector |
			classIsMeta := false.
			(methodEntry fileName beginsWith: aClassEntry fileName , 'class' , self slash)
				ifTrue: [ classIsMeta := true ].
			fileStream := self zipEntryContents: methodEntry.
			category := fileStream nextLine.
			source := fileStream upToEnd.
			selector := self methodSelectorFor: source.
			timestamp := methodProperties
				at:
					(classIsMeta
						ifTrue: [ 'class' ]
						ifFalse: [ 'instance' ])
				ifPresent: [ :map | map at: selector asString ifAbsent: [  ] ].	"Issue 33: https://github.com/dalehenrich/filetree/issues/33"
			timestamp ifNil: [ timestamp := self info author , ' ' , self info date mmddyyyy , ' ' , self info time print24 ].
			definitions
				add:
					(MCMethodDefinition
						className: className
						classIsMeta: classIsMeta
						selector: selector
						category: category
						timeStamp: timestamp
						source: source) ]! !

!MCFileTreeGitStReader methodsFor: 'accessing' stamp: 'ThierryGoubier 6/2/2013 18:31'!
commitID
	^ commitID! !

!MCFileTreeGitStReader methodsFor: 'accessing' stamp: 'ThierryGoubier 6/2/2013 18:31'!
commitID: aString
	commitID := aString! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 6/16/2013 12:29'!
definitionFromEntry: aZipFileMember
	| defs reader |
	reader := MCStReader on: (self zipEntryContents: aZipFileMember).
	(defs := reader definitions) size <= 1
		ifFalse: [ self error: 'we should be writing exactly 1 definition per file' ].
	defs size = 0
		ifTrue: [ ^ nil ].
	^ defs first! !

!MCFileTreeGitStReader methodsFor: 'private' stamp: 'ThierryGoubier 6/4/2013 16:39'!
definitionFromFile: directoryEntry inDirectory: aDirectory
	self shouldNotImplement! !

!MCFileTreeGitStReader methodsFor: 'testing' stamp: 'ThierryGoubier 6/4/2013 15:30'!
hasMonticelloMetadata
	"Two cases: we have either a minimal archive containing a version member or a full archive containing the metadata."
	
	"Try to avoid loading the full package :)"
	
	"Since we use a hack to avoid loading the full archive, consider this true."
	
	^ true! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 6/25/2013 14:42'!
loadDefinitionFor: aMCDefinition
	"Load the definition in this commit for that MCDefinition. Return nil if it is empty (effect of git tracking renames :))"

	| classPropertiesDict methodPropertiesDict entries classEntry |
	definitions := OrderedCollection new.
	self zipForDefinition: aMCDefinition.
	classEntry := packageDirectory , self slash , aMCDefinition className , '.class/'.
	entries := self zip members select: [ :entry | entry isDirectory not and: [ entry fileName beginsWith: classEntry ] ].
	entries isEmpty
		ifTrue: [ ^ nil ].
	classPropertiesDict := Dictionary new.
	(entries detect: [ :entry | entry splitFileName last = 'properties.json' ] ifNone: [  ])
		ifNotNil: [ :propertyEntry | classPropertiesDict := MCFileTreeJsonParser parseStream: (self zipEntryContents: propertyEntry) ].
	methodPropertiesDict := Dictionary new.
	(entries detect: [ :entry | entry splitFileName last = 'methodProperties.json' ] ifNone: [  ])
		ifNotNil: [ :propertyEntry | methodPropertiesDict := MCFileTreeJsonParser parseStream: (self zipEntryContents: propertyEntry) ].
	self
		addMethodDefinitionsForClass: (classPropertiesDict at: 'name')
		methodProperties: methodPropertiesDict
		in: entries
		root: (self zip members detect: [ :entry | entry isDirectory and: [ entry fileName endsWith: classEntry ] ]).
	^ definitions! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 6/5/2013 09:55'!
loadDefinitions
	"Query the zip archive for the relevant data."

	| query |
	definitions := OrderedCollection new.
	query := self zip
		membersMatching: packageDirectory , self slash , self monticelloMetaDirName , self slash , 'categories*'.
	query notEmpty
		ifTrue: [ self addDefinitionFromEntry: query first ]
		ifFalse: [ definitions add: (MCOrganizationDefinition categories: {(self packageNameFromPackageDirectory)}) ].
	self addClassAndMethodDefinitions.
	query := self zip
		membersMatching: packageDirectory , self slash , self monticelloMetaDirName , self slash , 'initializers*'.
	query notEmpty
		ifTrue: [ self addDefinitionFromEntry: query first ]! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 6/5/2013 11:02'!
loadDependencies
	"Not entirely sure of how that works, but... We'll try."

	| dependencyDir |
	dependencyDir := packageDirectory , self slash , self monticelloMetaDirName , self slash , 'dependencies' , self slash.
	dependencies := OrderedCollection new.
	(self zip members select: [ :entry | entry isDirectory not and: [ entry fileName beginsWith: dependencyDir ] ])
		do: [ :entry | 
			| name |
			name := entry splitFileName last.
			dependencies
				add:
					(MCVersionDependency
						package: (MCPackage named: name)
						info: (self extractInfoFrom: (self parseMember: 'dependencies' , self slash , name))) ].
	dependencies := dependencies asArray! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 6/17/2013 16:49'!
loadVersionInfo: aString
	"If we manage to have a correct string, with some version info, get it there."
	
	| tokens |
	tokens := [ self scanner scan: (ReadStream on: aString) ]
		on: Error
		do: [ #() ].
	^ info := self extractInfoFrom: (self associate: tokens)! !

!MCFileTreeGitStReader methodsFor: 'accessing' stamp: 'ThierryGoubier 6/4/2013 15:20'!
packageDirectory: aDirectoryName
	packageDirectory := aDirectoryName! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 6/20/2013 15:37'!
parseMember: fileName
	"Parse a metadata member? Try to load directly the file, have a fallback in case it doesn't exist."

	"Todo: ensure we have the right encoding."

	| monticelloMetaDirectory c tokens tempZip |
	monticelloMetaDirectory := self fileUtils
		directoryFromPath: self monticelloMetaDirName
		relativeTo: (self fileUtils directoryFromPath: packageDirectory relativeTo: stream).
	monticelloMetaDirectory exists
		ifFalse: [ ^ self associate: #() ].
	c := PipeableOSProcess
		command: 'cd ' , monticelloMetaDirectory pathString , '; git archive --format=zip ' , commitID , ' ' , fileName.
	tempZip := ZipArchive new.
	tempZip readFrom: (ReadStream on: c output asByteArray).
	c succeeded
		ifFalse: [ self error: 'Git error: ' , c errorPipelineContents ].
	tokens := [ self scanner scan: (self zipEntryContents: (tempZip memberNamed: fileName)) ]
		on: Error
		do: [ #() ].
	^ self associate: tokens! !

!MCFileTreeGitStReader methodsFor: 'private' stamp: 'ThierryGoubier 6/5/2013 09:55'!
slash
	^ String with: self fileUtils pathNameDelimiter! !

!MCFileTreeGitStReader methodsFor: 'accessing' stamp: 'ThierryGoubier 6/4/2013 15:51'!
zip
	"Get the zip based on the commitID."

	| c |
	zip isNil
		ifTrue: [ 
			zip := ZipArchive new.
			c := PipeableOSProcess
				command: 'cd ' , stream pathString , '; git archive --format=zip ' , commitID , ' ' , packageDirectory.
			zip readFrom: (ReadStream on: c output asByteArray).
			c succeeded
				ifFalse: [ self error: 'Git error: ' , c errorPipelineContents ] ].
	^ zip! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 6/20/2013 16:20'!
zipEntryContents: aZipFileMember
	"Try to get the correct encoding for UTF8."

	"Try to return a stream on a text!!"

	"TODO: replace by aZipFileMember>>contentsStreamFromEncoding: once issue 10978 is backported to 2.0."

	^ ReadStream on: ((TextConverter newForEncoding: 'utf8') convertFromSystemString: aZipFileMember contents)! !

!MCFileTreeGitStReader methodsFor: 'accessing' stamp: 'ThierryGoubier 6/25/2013 11:37'!
zipForDefinition: aMCDefinition
	"We suppose we are trying to load a specific method definition. Recover the relevant archive as needed."

	"Ask in the archive, the method file, and the two properties file."

	| c classDirectory methodFile methodPropertiesFile classPropertiesFile |
	self assert: aMCDefinition isMethodDefinition.
	zip := ZipArchive new.
	classDirectory := packageDirectory , self slash , aMCDefinition className
		,
			(aMCDefinition isExtensionMethod
				ifTrue: [ '.extension' ]
				ifFalse: [ '.class' ]).
	methodFile := classDirectory , self slash
		,
			(aMCDefinition classIsMeta
				ifTrue: [ 'class' ]
				ifFalse: [ 'instance' ]) , self slash , (MCFileTreeStCypressWriter new fileNameForSelector: aMCDefinition selector)
		, '.st'.
	methodPropertiesFile := classDirectory , self slash , 'methodProperties.json'.
	classPropertiesFile := classDirectory , self slash , 'properties.json'.
	c := PipeableOSProcess
		command:
			'cd ' , stream pathString , '; git archive --format=zip ' , commitID , ' ' , methodFile , ' ' , methodPropertiesFile
				, ' ' , classPropertiesFile.
	zip readFrom: (ReadStream on: c output asByteArray).
	c succeeded
		ifFalse: [ self error: 'Git error: ' , c errorPipelineContents ].
	^ zip! !

MCFileTreeRepository subclass: #MCFileTreeGitRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Git'!

!MCFileTreeGitRepository commentStamp: '<historical>' prior: 0!
A MCFileTreeGitRepository adds git commands via OSProcess when saving a package in a filetree.
Uses gitfiletree: as a protocol (:().

Commands done so far:
git add name.package; git commit ...
git add .filetree
	when creating the repository
git archive
	on a specific commit, retrieve the package (or part of it)

Git commands of interest:

git status
	To test if we are really in a git repo and that the git command works.
git log --follow --pretty="%H" --no-notes MonticelloFileTree-Git.package/monticello.meta/version
	To get all the commitIDs with all the versions of the packages.
git archive 5562cc769a36b0cdf29dbcb53aa84704b72ce60d MonticelloFileTree-Git.package/monticello.meta/version | tar x -O
	Will dump the version file for that specific commit on the command line, so that we may recover all we need from it.
	
Implementation. Get all commits hashes related to a package versions. Then get all version files to recover the data. Store the version and hash in the listing in the GUI... Now, about loading it... Change the reader to bring along the hash for that version, and use the git archive trick to load each package as a zip.

Has now a Gofer interface. Tries to correctly load utf8 data from the zip archive. Cuts the runtime by two.

[MCFileTreeGitRepository allInstances first cacheAllFileNamesDuring: [  Gofer new url: 'gitfiletree:///home/thierry/src/GitFileTree/AltBrowser'; version: 'ConfigurationOfAltBrowser-ThierryGoubier.1'; load]] timeToRun
!

!MCFileTreeGitRepository classSide methodsFor: 'instance creation' stamp: 'ThierryGoubier 5/29/2013 16:19'!
description
	^ 'gitfiletree://'! !

!MCFileTreeGitRepository classSide methodsFor: 'documentation' stamp: 'ThierryGoubier 6/25/2013 16:03'!
docForGitLogAttempts2
	"Some of this code is to test various options."

	"Testing a solution encapsulating the object and the commitID in one go."
	
	"MCFileTreeGitRepository docForGitLogAttempts2"
	
	"timeToRun: 5135 (x2 sur l'ancienne version). Mais timeToRun: 1862 sur la partie OSProcess seule. timeToRun on splitting then scanning the info: 3818. timeToRun for splitting: 392. Longuest bit is the scan."
	
	"Minimizing the scan: 474 to split and scan :) Removing the error handling... 450. Result is correct."

	| c commitIDs allNames tag uuid |
	tag := '^--^'.
	uuid := UUID new printString.
	c := PipeableOSProcess
		command:
			'cd /home/thierry/src/gitfiletree/AltBrowser/Alt-Browser.package/monticello.meta/; for i in `git log --pretty="%H" version`; do echo "'
				, tag , '"; echo $i; echo "' , tag , '"; echo `git show $i:./version`; done'.
	commitIDs := ReadStream on: c output.
	allNames := OrderedCollection new.
	c succeeded
		ifFalse: [ self error: 'Git error: ' , c errorPipelineContents ].
	self assert: (commitIDs match: tag).
	^ [ [ commitIDs atEnd ]
		whileFalse: [ 
			| commitID rawInfo info |
			commitID := commitIDs upToAll: tag.
			rawInfo := (ReadStream on: (commitIDs upToAll: tag)) upToAll: ''' message '''.
			info := "["
			((MCFileTreeGitStReader on: '/home/thierry/src/GitFileTree/AltBrowser' asFileReference fileName: 'Alt-Browser.package')
				commitID: commitID) loadVersionInfo: rawInfo , ''' message '''' id ''', uuid, ''' date '''' time '''' author '''' ancestors () stepChildren ())' "]
				on: KeyNotFound
				do: [ nil ]".
			info notNil
				ifTrue: [ allNames add: (GitFileTreePackageEntry on: 'Alt-Browser.package' id: commitID info: info) ] ].
	allNames ] timeToRun! !

!MCFileTreeGitRepository classSide methodsFor: 'accessing' stamp: 'ThierryGoubier 6/3/2013 15:30'!
monticelloMetaDirName
    ^ MCFileTreeStCypressWriter monticelloMetaDirName! !

!MCFileTreeGitRepository classSide methodsFor: 'instance creation' stamp: 'ThierryGoubier 6/1/2013 11:42'!
parseEntry: aPackageEntry extension: extension
	"picked up from GoferVersionReference>>parseName:"

	| basicName package branch author versionNumber packageName string |
	string := aPackageEntry printString.
	basicName := string last isDigit
		ifTrue: [ string ]
		ifFalse: [ (string copyUpToLast: $.) copyUpTo: $( ].
	package := basicName copyUpToLast: $-.
	(package includes: $.)
		ifFalse: [ branch := '' ]
		ifTrue: [ 
			branch := '.' , (package copyAfter: $.).
			package := package copyUpTo: $. ].
	author := (basicName copyAfterLast: $-) copyUpToLast: $..
	versionNumber := (basicName copyAfterLast: $-) copyAfterLast: $..
	(versionNumber notEmpty and: [ versionNumber allSatisfy: [ :each | each isDigit ] ])
		ifTrue: [ versionNumber := versionNumber asNumber ]
		ifFalse: [ versionNumber := 0 ].
	packageName := package , branch.
	^ {packageName.
	author.
	versionNumber.
	aPackageEntry}! !

!MCFileTreeGitRepository classSide methodsFor: 'instance creation' stamp: 'ThierryGoubier 5/31/2013 23:21'!
parseName: aString extension: extension
	"picked up from GoferVersionReference>>parseName:"

	| basicName package branch author versionNumber packageName |
	basicName := aString last isDigit
		ifTrue: [ aString ]
		ifFalse: [ (aString copyUpToLast: $.) copyUpTo: $( ].
	package := basicName copyUpToLast: $-.
	(package includes: $.)
		ifFalse: [ branch := '' ]
		ifTrue: [ 
			branch := '.' , (package copyAfter: $.).
			package := package copyUpTo: $. ].
	author := (basicName copyAfterLast: $-) copyUpToLast: $..
	versionNumber := (basicName copyAfterLast: $-) copyAfterLast: $..
	(versionNumber notEmpty and: [ versionNumber allSatisfy: [ :each | each isDigit ] ])
		ifTrue: [ versionNumber := versionNumber asNumber ]
		ifFalse: [ versionNumber := 0 ].
	packageName := package , branch.
	^ {packageName.
	author.
	versionNumber.
	basicName}! !

!MCFileTreeGitRepository methodsFor: 'accessing' stamp: 'ThierryGoubier 6/16/2013 23:17'!
allFileNames
	"Switch to caching for speed."
	
	cacheFileNames == true ifFalse: [ ^ self loadAllFileNames ].
	^ allFileNames ifNil: [
			allFileNames := self loadAllFileNames]! !

!MCFileTreeGitRepository methodsFor: 'i/o' stamp: 'ThierryGoubier 6/26/2013 16:29'!
basicStoreVersion: aVersion
	"Do a filetree store followed by the right git commands."

	"cd in the repository; do git add the package to register the new files, then git commit the package with the package message."

	"A git commit without any change return a non zero status without any error message."

	| c packageDirectoryString |
	super basicStoreVersion: aVersion.
	packageDirectoryString := (self class parseName: aVersion info name) first , self packageExtension.
	c := PipeableOSProcess
		command:
			'cd ' , directory pathString , '; git add ' , packageDirectoryString , '; git commit -m "'
				, (self escapeForShell: ((TextConverter newForEncoding: 'utf8') convertToSystemString: aVersion info message))
				, '" -- ' , packageDirectoryString.
	c output.
	c succeeded
		ifFalse: [ 
			| serr |
			serr := c errorPipelineContents.
			serr notEmpty
				ifTrue: [ self error: 'Git error: ' , serr ] ]! !

!MCFileTreeGitRepository methodsFor: 'accessing' stamp: 'ThierryGoubier 6/17/2013 10:23'!
cacheAllFileNames
	"Switch to caching for speed."

	cacheFileNames = true! !

!MCFileTreeGitRepository methodsFor: 'private' stamp: 'ThierryGoubier 6/26/2013 16:29'!
escapeForShell: aString
	^ aString class
		new: aString size
		streamContents: [ :stream | 
			| readStream char |
			readStream := aString readStream.
			[ readStream atEnd ]
				whileFalse: [ 
					char := readStream next.
					char = $"
						ifTrue: [ stream nextPut: $\ ].
					stream nextPut: char ] ]! !

!MCFileTreeGitRepository methodsFor: 'private' stamp: 'ThierryGoubier 6/1/2013 11:20'!
fileDirectoryOn: aGitFileTreePackageEntry
	"We are dealing with GitFileTreePackageEntry objects, not FileReferences."

	^ self fileUtils directoryFromPath: aGitFileTreePackageEntry packageDirectory relativeTo: self directory! !

!MCFileTreeGitRepository methodsFor: 'git querying' stamp: 'ThierryGoubier 6/26/2013 22:22'!
gitVersionsForDefinition: aMCDefinition in: aPackageName
	"Query git for all the versions related to that method definition."

	| c commitIDs fileDirectory methodFile |
	fileDirectory := self fileUtils
		directoryFromPath:
			aMCDefinition className
				,
					(aMCDefinition isExtensionMethod
						ifTrue: [ '.extension' ]
						ifFalse: [ '.class' ])
		relativeTo: (self fileUtils directoryFromPath: aPackageName , '.package' relativeTo: directory).
	methodFile := (aMCDefinition classIsMeta
		ifTrue: [ 'class' ]
		ifFalse: [ 'instance' ]) , '/' , (MCFileTreeStCypressWriter new fileNameForSelector: aMCDefinition selector) , '.st'.
	fileDirectory exists
		ifFalse: [ NotFound signalFor: fileDirectory ].
	c := PipeableOSProcess command: 'cd ' , fileDirectory pathString , '; git log --pretty="%H" ' , methodFile.
	commitIDs := ReadStream on: c output.
	c succeeded
		ifFalse: [ NotFound signalFor: c errorPipelineContents ].
	^ commitIDs contents substrings! !

!MCFileTreeGitRepository methodsFor: 'git querying' stamp: 'ThierryGoubier 6/23/2013 12:04'!
gitVersionsForPackage: aDirectoryEntry
	"Query git for all the versions related to that directory entry"

	"Fact: if a package is updated without increasing the version, we don't see it with this code. If we see it (different git query), then we have multiple packages with the same version and not real way to distinguish them in the GUI."
	
	"As of now, see only changes to version."

	| c commitIDs monticelloMetaDirectory |
	monticelloMetaDirectory := self fileUtils
		directoryFromPath: self class monticelloMetaDirName
		relativeTo: (self fileUtils directoryFromPath: aDirectoryEntry relativeTo: directory).
	c := PipeableOSProcess command: 'cd ' , monticelloMetaDirectory pathString , '; for i in `git log --pretty="%H" version`; do echo "'
				, self tag , '"; echo $i; echo "' , self tag , '"; echo `git show $i:./version`; done'.
	commitIDs := ReadStream on: c output.
	c succeeded
		ifFalse: [ self error: 'Git error: ' , c errorPipelineContents ].
	^ commitIDs! !

!MCFileTreeGitRepository methodsFor: 'accessing' stamp: 'ThierryGoubier 6/16/2013 23:00'!
goferVersionFrom: aVersionReference
	"gitfiletree:// provides all version names as found by git."

	"Should be part of gofer, but isn't."

	"Since gofer is text based, we can't give him git entries, but strings and those are then rematched to get the right entry."

	| mcR packageName |
	packageName := (self class parseName: aVersionReference name) first , self packageExtension.
	mcR := MCFileTreeGitStReader on: directory fileName: packageName.
	mcR commitID: (self allFileNames detect: [ :e | e name = aVersionReference name ]) commitID.
	^ mcR version! !

!MCFileTreeGitRepository methodsFor: 'intialize-release' stamp: 'ThierryGoubier 6/17/2013 07:02'!
initialize
	super initialize.
	"cacheFileNames := true"! !

!MCFileTreeGitRepository methodsFor: 'accessing' stamp: 'ThierryGoubier 6/25/2013 16:06'!
loadAllFileNames
	"When reading the file names, generate GitFileTreePackageEntry objects with the path and the info."

	| allNames entries commitIDs uuid |
	uuid := UUID new printString.
	allNames := OrderedCollection new.
	entries := self directory entries select: [ :entry | entry isDirectory and: [ self canReadFileNamed: entry name ] ].
	'loading packages versions from git'
		displayProgressFrom: 0
		to: entries size
		during: [ :bar | 
			1 to: entries size do: [ :i | 
				(entries at: i)
					in: [ :entry | 
						commitIDs := self gitVersionsForPackage: entry name.
						(commitIDs match: self tag)
							ifFalse: [ self error: 'Unable to find tag in input' ].
						[ commitIDs atEnd ]
							whileFalse: [ 
								| commitID rawInfo info |
								commitID := (commitIDs upToAll: self tag) trimBoth.
								rawInfo := (ReadStream on: (commitIDs upToAll: self tag)) upToAll: ''' message '''.
								info := [ 
								((MCFileTreeGitStReader on: directory fileName: entry name) commitID: commitID)
									loadVersionInfo:
										rawInfo , ''' message '''' id ''' , uuid , ''' date '''' time '''' author '''' ancestors () stepChildren ())' ]
									on: KeyNotFound
									do: [ nil ].
								info notNil
									ifTrue: [ allNames add: (GitFileTreePackageEntry on: entry name id: commitID info: info) ] ] ].
				bar value: i ] ].
	^ allNames! !

!MCFileTreeGitRepository methodsFor: 'interface' stamp: 'ThierryGoubier 6/1/2013 21:14'!
morphicOpen: aWorkingCopy
	(MCFileTreeGitRepositoryInspector repository: self workingCopy: aWorkingCopy) show! !

!MCFileTreeGitRepository methodsFor: 'accessing' stamp: 'ThierryGoubier 6/1/2013 11:44'!
packageDescriptionFromPackageEntry: packageEntry
	| filename extension packageDirectory |
	packageDirectory := self fileDirectoryOn: packageEntry.
	filename := self fileUtils current directoryName: packageDirectory.
	extension := filename copyFrom: (filename lastIndexOf: $.) to: filename size.
	^ ((self packageExtension ~= '.package'
		and: [ 
			(self fileUtils filePathExists: 'version' relativeTo: packageDirectory)
				and: [ self fileUtils filePathExists: 'package' relativeTo: packageDirectory ] ])
		or: [ 
			| dir |
			dir := self fileUtils directoryFromPath: MCFileTreeStCypressWriter monticelloMetaDirName relativeTo: packageDirectory.
			self fileUtils directoryExists: dir ])
		ifTrue: [ self class parseEntry: packageEntry extension: extension ]
		ifFalse: [ 
			{(filename copyFrom: 1 to: (filename lastIndexOf: $.) - 1).
			'cypress'.
			1.
			filename} ]! !

!MCFileTreeGitRepository methodsFor: 'accessing' stamp: 'ThierryGoubier 6/1/2013 11:43'!
packageDescriptionsFromReadableFileNames
	^ (self readableFileNames
		select: [ :packageEntry | self fileUtils directoryExists: (self fileDirectoryOn: packageEntry) ])
			collect: [ :packageEntry | self packageDescriptionFromPackageEntry: packageEntry ]! !

!MCFileTreeGitRepository methodsFor: 'accessing' stamp: 'ThierryGoubier 6/2/2013 21:41'!
readableFileNames
	"Updated for gitfiletree use of packageEntry, not file names."

	^ self allFileNames! !

!MCFileTreeGitRepository methodsFor: 'private' stamp: 'ThierryGoubier 6/23/2013 12:01'!
tag
	^ '^--^'! !

!MCFileTreeGitRepository methodsFor: 'accessing' stamp: 'ThierryGoubier 6/5/2013 11:37'!
versionFromFileNamed: aGitFileTreePackageEntry
	"In the GUI, we should have version names (contributeur, version number, branch) for aString instead of the filetree package directory itself. Handle both cases for the moment."

	"Will change when we will be reading the version directly from the git."

	"Add a bit of debugging. Here aString should be a GitFileTreePackageEntry in fact (but probably it isn't)."

	| mcR packageName |
	self assert: aGitFileTreePackageEntry class == GitFileTreePackageEntry.
	packageName := (self class parseName: aGitFileTreePackageEntry info name) first , self packageExtension.
	mcR := MCFileTreeGitStReader on: directory fileName: packageName.
	mcR commitID: aGitFileTreePackageEntry commitID.
	^ mcR version! !

!MCFileTreeGitRepository methodsFor: 'accessing' stamp: 'ThierryGoubier 6/16/2013 22:58'!
versionNameFromFileName: ea
	^ ea name! !

!MCFileTreeGitRepository methodsFor: 'i/o' stamp: 'ThierryGoubier 6/11/2013 14:23'!
writeRepositoryProperties
	| c s |
	super writeRepositoryProperties.
	c := PipeableOSProcess
		command:
			'cd ' , directory pathString , '; git add .filetree; git commit -m "gitfiletree repository properties" -- .filetree'.
	s := c output.
	c succeeded
		ifTrue: [ 
			| serr |
			serr := c errorPipelineContents.
			serr notEmpty
				ifTrue: [ c self error: 'Git error: ' , serr , ': ' , s ] ]! !

Object subclass: #GitFileTreePackageEntry
	instanceVariableNames: 'packageDirectory info commitID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Git'!

!GitFileTreePackageEntry commentStamp: '<historical>' prior: 0!
A GitFileTreePackage represent a package in a gitfiletree://repository

Instance Variables
	info:		<MCVersion>
	packageDirectory:		<FileReference>!

!GitFileTreePackageEntry classSide methodsFor: 'instance creation' stamp: 'ThierryGoubier 6/2/2013 21:35'!
on: aPackageDirectory id: aCommitID info: info
	^ self new
		packageDirectory: aPackageDirectory;
		info: info;
		commitID: aCommitID! !

!GitFileTreePackageEntry classSide methodsFor: 'instance creation' stamp: 'ThierryGoubier 5/31/2013 22:37'!
on: aPackageDirectory info: info
	^ self new
		packageDirectory: aPackageDirectory;
		info: info! !

!GitFileTreePackageEntry methodsFor: 'accessing' stamp: 'ThierryGoubier 6/2/2013 21:36'!
commitID
	^ commitID! !

!GitFileTreePackageEntry methodsFor: 'accessing' stamp: 'ThierryGoubier 6/2/2013 21:36'!
commitID: aString
	commitID := aString! !

!GitFileTreePackageEntry methodsFor: 'accessing' stamp: 'ThierryGoubier 5/31/2013 22:34'!
info
	^ info! !

!GitFileTreePackageEntry methodsFor: 'accessing' stamp: 'ThierryGoubier 5/31/2013 22:35'!
info: aMCVersion
	info := aMCVersion! !

!GitFileTreePackageEntry methodsFor: 'accessing' stamp: 'ThierryGoubier 6/16/2013 22:48'!
name
	^ info name! !

!GitFileTreePackageEntry methodsFor: 'accessing' stamp: 'ThierryGoubier 5/31/2013 22:34'!
packageDirectory
	^ packageDirectory! !

!GitFileTreePackageEntry methodsFor: 'accessing' stamp: 'ThierryGoubier 5/31/2013 22:35'!
packageDirectory: aPackageDirectory
	packageDirectory := aPackageDirectory! !

!GitFileTreePackageEntry methodsFor: 'printing' stamp: 'ThierryGoubier 5/31/2013 22:41'!
printString
	"Get the name of the package."

	^ info name! !

MCFileRepositoryInspector subclass: #MCFileTreeGitRepositoryInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Git'!

!MCFileTreeGitRepositoryInspector commentStamp: '<historical>' prior: 0!
This code is there to adapt the GUI to the FileTreeGitRepository.!

!MCFileTreeGitRepositoryInspector methodsFor: 'accessing' stamp: 'ThierryGoubier 6/1/2013 22:45'!
version
	^ version
		ifNil: [ 
			Cursor wait
				showWhile: [ version := repository versionFromFileNamed: (versions detect: [ :e | e last name = selectedVersion ]) last ].
			version ]! !

!MCFileTreeGitRepositoryInspector methodsFor: 'accessing' stamp: 'ThierryGoubier 6/1/2013 23:13'!
versionHighlight: aPackageEntry
	"Changed to take in account the fact the version name is in the package entry."
	
	"Should correct a bug in FileTree display: current package is allways in bold."

	| verName |
	self assert: aPackageEntry class == GitFileTreePackageEntry.
	inherited ifNil: [ inherited := #() ].
	verName := aPackageEntry info name.
	^ Text
		string: verName
		attribute:
			(TextEmphasis new
				emphasisCode:
					((loaded includes: verName)
						ifTrue: [ 4	"underlined" ]
						ifFalse: [ 
							(inherited includes: verName)
								ifTrue: [ 0 ]
								ifFalse: [ 1	"bold" ] ]))! !

!MCFileTreeGitRepositoryInspector methodsFor: 'accessing' stamp: 'ThierryGoubier 6/5/2013 13:05'!
versionInfo
	^ self version info! !
